/*
   Copyright (C) 1988, 1989, 1990 The GAP Development Company

   All Rights Reserved
   
   This source code is considered to be confidential information proprietary
   to The GAP Development Company.  It is prohibited and punishable by law
   to use or duplicate any part of this information without express
   written consent from GAP Development Company.

   This source code is being provided for informational purposes only.


   UTIDOOR.C

   UTI module that provides the name and time remaining of the user
   currently in a door.

   This program should be placed in the GAP DEFAULT directory as
   that is where it expects to read DOOR.SYS.


   To compile : cl /FPa utidoor.c        (will compile and link)

*/

#define LINT_ARGS

#define MULTIUSER
//#define SINGLEUSER

#pragma check_stack(off)
#pragma pack(1)

#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <fcntl.h>
#include <sys\types.h>
#include <sys\stat.h>
#include <share.h>
#include <io.h>

   char username   [40];                  // caller's name
   char anystring [250];                  // garbage collector


   /**********************************************************************/
   /* A shortened version of GAPBBS.CNF.  All we need is the path to the  /
   /* main directory.                                                     /
   /**********************************************************************/

   struct CONFIG
      {
      char sysname       [16];            // sysop name                   
      char syspas        [13];            // sysop local password         
      int sys_read;                       // level to read private mail   
      int sys_edit;                       // level to edit msg headers    
      int overwrite;                      // level to overwrite uploads   
      int sys_level1;                     // level to view callers log    
      int sys_level2;                     // level to view users          
      int sys_level3;                     // level to view a file         
      int sys_level4;                     // level to maintain user file  
      int sys_level5;                     // level to exit to dos         
      int sys_level5a;                    // level to execute dos funcs   
      int sys_level6;                     // level to view other nodes    
      int sys_level6a;                    // level to log off other node  
      int sys_level7;                     // level to recover messages    

      char answer         [8];            // questionnaire answers file   
      char caller         [7];            // caller file                  
      char doors          [9];            // doors.dat file               
      char forumdat       [9];            // name of forum (.DAT) file    
      char entry_news     [8];            // news file                    
      char exit_news      [8];            // exit news file               
      char pwrd           [9];            // time / security file         
      char fsec           [9];            // files security file          
      char msgs           [9];            // messages file                
      char new_welcome    [8];            // new user welcome file        
      char answer_regis   [8];            // not used                     
      char remote         [9];            // remote.sys file              
      char tcan           [9];            // trashcan file                
      char users          [9];            // users file                   
      char welcome        [8];            // welcome file                 

      char uploads        [26];           // upload directory             
      char gen            [26];           // general directory            
      char maindir        [26];           // main directory               
      }config;


   struct USER
      {
      char fname     [16];                // first name
      char lname     [16];                // last name
      char handle    [16];                // handle for MPART!
      char city      [30];                // city, state
      char passwd    [13];                // password
      char hphone    [14];                // homephone
      char bphone    [14];                // work/data phone
      char subscribe  [9];                // subscription date
      int  level;                         // security - up to 255
      long timeson;                       // times on - over 2 billion
      char lastdate   [9];                // last date on - mm/dd/yy
      char lasttime   [6];                // last time on - hh:mm
      long ttlmins;                       // total mins on - 2+ billion
      int  minutes;                       // # of minutes last call
      long uploads;                       // # of uploads - 2+ billion
      long downloads;                     // # of downloads - 2+ billion
      long ttlbytes;                      // total bytes downloaded
      long curbytes;                      // bytes downloaded this call
      char private;                       // allow on private nodes?
      char protocol   [3];                // transfer protocol
      int  page;                          // page length - to 23
      char lastdir    [9];                // date last looked at dir
      long lastmsg;                       // last message read - 99999 max
      long mesread;                       // total msgs read - 2+ billion
      long mesleft;                       // total messages left
      long forum_joined;                  // total forums joined
      long blts_read;                     // total bulletins read
      long doors_opened;                  // total doors opened
      int  lastconf;                      // last forum in
      char expert;                        // novice = N, expert = Y
      char sysop;                         // sysop record, 1 = Y, 0 = N
      long upbytes;                       // total bytes uploaded
      int curdown;                        // total files downed today
      char birthdate  [9];                // user's birthdate
      char comment   [51];                // comment about user
      long forum1;                        // forum membership  1  - 30
      long forum2;                        // forum membership  31 - 60
      long forum3;                        // forum membership  61 - 90
      long fmes      [90];                // last message read each forum
      }user;                              // 652 bytes


   static void pascal get_sysop(void );
   static char *trim(char *str);


void main(argc,argv)
int argc;
char *argv[];
{
   register int r;
   char port;                             // com port being used
   char baud       [12];                  // caller's baud rate
   char dte        [12];                  // actual baud rate
   char seconds    [12];                  // seconds left
   char hismail    [60];                  // path to UTI DOOR file

   FILE *fp;
   FILE *doorsys;

   strcpy(hismail,(argc < 2) ? "UTIDOOR.TXT" : argv[1]);

   if ( (fp = fopen(hismail,"w+t")) == NULL)         // open UTIDOOR.TXT
      exit (-1);

   if ( (doorsys = fopen("DOOR.SYS","rt")) == NULL)  // open DOOR.SYS
      {
      fclose(fp);
      exit (-1);
      }

   fgets(anystring,99,doorsys);           // get com port
   port = anystring[3];                   // port number

   fgets(baud,99,doorsys);                // caller's baud rate

   for (r = 0; r < 2; r++)
      fgets(anystring,99,doorsys);        // get garbage
 
   fgets(dte,99,doorsys);                 // actual baud rate

   for (r = 0; r < 4; r++)
      fgets(anystring,99,doorsys);        // get garbage

   fgets(username,99,doorsys);            // get user's name

   for (r = 0; r < 7; r++)
      fgets(anystring,99,doorsys);        // get garbage

   fgets(seconds,99,doorsys);             // get seconds left

   fclose(doorsys);                       // Close door.sys file

   if ((strncmp(username,"SYSOP",5)) == 0)  // is this the sysop?
      get_sysop();                        // yes, go read sysop record

   fputs(username,fp);                    // write caller's name
   fputs(baud,fp);                        // write baud rate
   fprintf(fp,"%c\n",port);               // write port number
   fputs(dte,fp);                         // write dte rate
   fputs(seconds,fp);                     // write seconds left

   fclose(fp);

   exit(0);
}

   // if the caller is the Sysop, reads the 1st record in the
   // user file to get the sysop's log in names.

static void pascal get_sysop()
{
   register int fp;

#ifdef MULTIUSER
   fp = sopen("GAPBBS.CNF",O_RDONLY | O_BINARY,SH_DENYNO);
#else
   fp = open("GAPBBS.CNF",O_RDONLY | O_BINARY);
#endif

   if (fp == -1)
      {
      cputs("\r\n\r\nCannot Open GAPBBS.CNF.\r\n");
      exit(1);
      }

   read(fp,(char *) &config,sizeof(config));
   close(fp);

   trim(config.maindir);                  // get rid of spaces
   trim(config.users);

   sprintf(anystring,"%s\\%s.DAT",config.maindir,config.users);

#ifdef MULTIUSER
   fp = sopen(anystring,O_RDONLY|O_BINARY,SH_DENYNO);   // open the user file
#else
   fp = open(anystring,O_RDONLY|O_BINARY);              // open the user file
#endif

   if (fp == -1)                          // cant open file
      {
      cputs("\r\nCannot Open USER File!\r\n");
      exit(1);
      }

   lseek(fp,(long) sizeof (user),SEEK_SET);      // sysop record is 1st one
   read(fp,(char *) &user,sizeof(user));
   close(fp);

   trim(user.fname);                      // get rid of spaces
   trim(user.lname);

   sprintf(username,"%s %s\n",user.fname,user.lname);
}


#pragma loop_opt(on)

static char *trim(str)
char *str;
{
   char *ptr;
   
   ptr = str + strlen(str) -1;
  
   while(ptr >= str && *ptr == ' ')
      *ptr-- = 0;

   return(str);
}

